################################################################################
# This entrypoint is also used in the empty hud-sdk package.
# Do not import any modules from the hud-sdk package here.
################################################################################

import argparse
import os
import shutil
import sys
from typing import Optional

from .utils import (
    log_to_user,
    set_config_path,
    set_key,
    set_service,
)


def find_command_executable(command: str) -> Optional[str]:
    if os.path.isfile(command):
        return command
    return shutil.which(command)


def add_path_to_pythonpath(path: str) -> None:
    python_path = os.environ.get("PYTHONPATH", "")
    python_path = f"{path}{os.path.pathsep}{python_path}"
    os.environ["PYTHONPATH"] = python_path


def set_hud_auto_init_dir() -> None:
    auto_init_dir = os.path.dirname(os.path.abspath(__file__))
    add_path_to_pythonpath(auto_init_dir)


def add_opentelemetry_to_path_if_needed() -> None:
    """
    Opentelmetry's auto_instrumentation removes themselves from the PYTHONPATH after running since they don't want to run on subprocesses.
    However, if a user runs `opentelemetry-instrument hud-run python main.py`, their instrumentation will only run in the `hud-run` process and not in the `python main.py` process.
    This function adds them back to the PYTHONPATH so that their instrumentation will run in the `python main.py` process.
    """
    try:
        otel_auto_init_module = sys.modules.get(
            "opentelemetry.instrumentation.auto_instrumentation"
        )
        if otel_auto_init_module:
            file_path = otel_auto_init_module.__file__
            if not file_path:
                return
            otel_auto_init_dir = os.path.dirname(file_path)
            if otel_auto_init_dir in os.environ.get("PYTHONPATH", ""):
                return
            if not os.path.exists(os.path.join(otel_auto_init_dir, "sitecustomize.py")):
                return
            add_path_to_pythonpath(otel_auto_init_dir)
    except Exception:
        pass


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--config",
        default=None,
        help="Path to the config file",
    )
    parser.add_argument(
        "--key",
        default="",
        help="Hud API key",
    )
    parser.add_argument(
        "--service",
        default="",
        help="Hud service name",
    )
    parser.add_argument(
        "command", nargs=argparse.REMAINDER, help="Command to run (Required)"
    )

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        sys.exit(1)

    if args.config:
        set_config_path(args.config)

    if args.key:
        set_key(args.key)

    if args.service:
        set_service(args.service)

    add_opentelemetry_to_path_if_needed()
    set_hud_auto_init_dir()

    command_found = find_command_executable(args.command[0])
    if not command_found:
        # In case you want to change this log, please edit it both here and in the users_logs.py file for docs purposes.
        log_to_user("[ERROR] Hud: Command executable not found.")
        sys.exit(1)

    os.execl(command_found, command_found, *args.command[1:])


if __name__ == "__main__":
    main()



################################################################################
# Shared utilities for auto_init entrypoints.
# This file can be imported by both hud_entrypoint.py and __main__.py
# Do not import any modules from the hud_sdk package here.
################################################################################

import os
import sys


def log_to_user(msg: str) -> None:
    print(msg, file=sys.stderr)


def set_config_path(config_path: str) -> None:
    os.environ["HUD_CONFIG_PATH"] = config_path


def set_key(key: str) -> None:
    os.environ["HUD_KEY"] = key


def set_service(service: str) -> None:
    os.environ["HUD_SERVICE"] = service



import argparse
import os
import runpy
import sys

from hud_sdk.auto_init.utils import (
    log_to_user,
    set_config_path,
    set_key,
    set_service,
)


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--config",
        default=None,
        help="Path to the config file",
    )
    parser.add_argument(
        "--key",
        default="",
        help="Hud API key",
    )
    parser.add_argument(
        "--service",
        default="",
        help="Hud service name",
    )
    parser.add_argument(
        "script",
        nargs=argparse.REMAINDER,
        help="Python script to run and its arguments (Required)",
    )

    args = parser.parse_args()

    if not args.script:
        parser.print_help()
        sys.exit(1)

    if args.config:
        set_config_path(args.config)

    if args.key:
        set_key(args.key)

    if args.service:
        set_service(args.service)

    try:
        from hud_sdk.auto_init.start_hud import start_hud

        start_hud()
    except Exception:
        pass

    script_path = args.script[0]
    if not script_path:
        log_to_user("[ERROR] Hud: No script path provided.")
        sys.exit(1)

    if not os.path.exists(script_path):
        log_to_user(f"[ERROR] Hud: Script file not found: {script_path}")
        sys.exit(1)

    if not os.path.isfile(script_path):
        log_to_user(f"[ERROR] Hud: Path is not a file: {script_path}")
        sys.exit(1)

    full_path = os.path.abspath(script_path)

    # Modify sys.argv to pass script arguments to the executed script
    sys.argv = args.script

    runpy.run_path(full_path, run_name="__main__")


if __name__ == "__main__":
    main()



import sys
import os

print(
    "Hud does not support this platform yet. The SDK has initiated a graceful shutdown. Your application remains unaffected. See the compatibility matrix for details: https://docs.hud.io/docs/hud-sdk-compatibility-matrix-for-python"
)


def run_previous_sitecustomize() -> None:
    auto_init_dir = os.path.dirname(__file__)
    if auto_init_dir not in sys.path:
        try:
            import sitecustomize  # type: ignore[import-not-found] # noqa: F401
        except ImportError:
            pass
        return

    index = sys.path.index(auto_init_dir)
    del sys.path[index]

    our_sitecustomize = sys.modules["sitecustomize"]
    del sys.modules["sitecustomize"]

    try:
        import sitecustomize  # noqa: F401
    except ImportError:
        sys.modules["sitecustomize"] = our_sitecustomize
    finally:
        sys.path.insert(index, auto_init_dir)


run_previous_sitecustomize()



from .version import version as __version__


print(
    "Hud does not support this platform yet. The SDK has initiated a graceful shutdown. Your application remains unaffected. See the compatibility matrix for details: https://docs.hud.io/docs/hud-sdk-compatibility-matrix-for-python"
)


def init_session(*args, **kwargs):
    pass


def register(*args, **kwargs):
    pass


def init(*args, **kwargs):
    pass


def set_hook(*args, **kwargs):
    pass

def set_failure(*args, **kwargs):
    pass

def set_context(*args, **kwargs):
    pass

class RegisterConfig:
    def __init__(self, *args, **kwargs):
        pass


__all__ = [
    "__version__",
    "RegisterConfig",
    "init_session",
    "register",
    "init",
    "set_hook",
    "set_failure",
    "set_context",
]



import os
from typing import Dict  # noqa: F401

from setuptools import setup

# Read version from hud_sdk/version.py since 'attr:' is not supported in setup.cfg in py35
version = {}  # type: Dict[str, str]
with open(os.path.join("src", "hud_sdk", "version.py")) as fp:
    exec(fp.read(), version)


setup(
    version=version["version"],
    entry_points={
        "console_scripts": [
            "hud-run = hud_sdk.auto_init.hud_entrypoint:main",
        ],
    },
)
